/***************************************************************************
   ktt.cpp  -  KTTApp

   begin     : 2002.11.08
   copyright : (C) 2002 by MiMaS
   email     : mimas@mimas.ceti.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qdir.h>
#include <qprinter.h>
#include <qpainter.h>
#include <qtextedit.h>
#include <qtooltip.h>
#include <qclipboard.h> 
#include <qradiobutton.h>

// include files for KDE
#include <kiconloader.h>
#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kmenubar.h>
#include <kstatusbar.h>
#include <kconfig.h>
#include <kstdaction.h>
#include <kio/netaccess.h>

// application specific includes
#include "ktt.h"                    
#include "kttview.h"
#include "kttdoc.h"
#include "kttsetupdlg.h"
#include "kttcodec.h"
#include "narmacil/exception.h"

// widgets and dialogs
#include "alertform.h"
#include "numberform.h"
#include "listitemform.h"


KTTApp::KTTApp(QWidget* , const char* name) : KMainWindow(0, name)
{
  config = kapp->config();

  doc = new KTTDoc(this);
  view = new KTTView(this);
  doc->addView(view);
  setCentralWidget(view);

  initStatusBar();
  initActions();
  createGUI();

  actFileSave->setEnabled(false);
  actEditCut->setEnabled(false);
  actEditCopy->setEnabled(false);
  actEditPaste->setEnabled(!(qApp->clipboard()->text().isEmpty()));
  actTransToTengwar->setEnabled(false);
  actTransToRoman->setEnabled(false);
  actUseTengwarFont->setChecked(true);
  
  narmacil = NULL;    // new CTranscription is in LoadNarmacilMode()
  
  readOptions();

  doc->newDocument();
  setDualCaption();
}

KTTApp::~KTTApp()
{
}

void KTTApp::initActions()
{
  actFileNew = KStdAction::openNew(this, SLOT(slotFileNew()), actionCollection(), "std_file_new");
  actFileOpenRoman = new KAction(i18n("Open Roman File..."), "fileopen", 0, this, SLOT(slotFileOpenRoman()), actionCollection(), "file_open_roman");
  actFileOpenTengwar = new KAction(i18n("Open Tengwar File..."), "fileopen", 0, this, SLOT(slotFileOpenTengwar()), actionCollection(), "file_open_tengwar");
  actFileOpenRecent = KStdAction::openRecent(this, SLOT(slotFileOpenRecent(const KURL&)), actionCollection(), "std_file_open_recent");
  actFileOpenRecent->setMaxItems(9);
  actFileSave = KStdAction::save(this, SLOT(slotFileSave()), actionCollection(), "std_file_save");
  actFileSaveAs = KStdAction::saveAs(this, SLOT(slotFileSaveAs()), actionCollection(), "std_file_save_as");
  actFilePrint = KStdAction::print(this, SLOT(slotFilePrint()), actionCollection(), "std_file_print");
  actFileQuit = KStdAction::quit(this, SLOT(slotFileQuit()), actionCollection(), "std_file_quit");

  actEditCut = KStdAction::cut(this, SLOT(slotEditCut()), actionCollection(), "std_edit_cut");
  actEditCopy = KStdAction::copy(this, SLOT(slotEditCopy()), actionCollection(), "std_edit_copy");
  actEditPaste = KStdAction::paste(this, SLOT(slotEditPaste()), actionCollection(), "std_edit_paste");
// next verion maybe
//  actInsertNumber = new KAction(i18n("Insert Tengwar &Number..."), 0, 0, this, SLOT(slotInsertNumber()), actionCollection(), "insert_number" );
  actInsertListItem = new KAction(i18n("Insert Tengwar List &Item..."), 0, 0, this, SLOT(slotInsertListItem()), actionCollection(), "insert_listitem" );
    
  actViewToolBarMain = new KToggleAction(i18n("Show Main Toolbar"), 0, this, SLOT(slotViewToolBarMain()), actionCollection(), "show_toolbar_main");
  actViewToolBarActions = new KToggleAction(i18n("Show Actions Toolbar"), 0, this, SLOT(slotViewToolBarActions()), actionCollection(), "show_toolbar_actions");
  actViewStatusBar = KStdAction::showStatusbar(this, SLOT(slotViewStatusBar()), actionCollection(), "std_show_statusbar");
  actUseTengwarFont = new KToggleAction(i18n("Use Tengwar Font"), 0, 0, this, SLOT(slotToggleTengwarFont()), actionCollection(), "use_tengwar_font" );

  actTransToTengwar = new KAction(i18n("Roman To &Tengwar"), "redo", CTRL+Key_T, this, SLOT(slotTransToTengwar()), actionCollection(), "roman_to_tengwar" );
  actTransToRoman = new KAction(i18n("Tengwar To &Roman"), "undo", CTRL+Key_R, this, SLOT(slotTransToRoman()), actionCollection(), "tengwar_to_roman" );
  actAutoTranscribe = new KToggleAction(i18n("Auto Transcribe"), "reload", 0, this, SLOT(slotTransAvailable()), actionCollection(), "auto_transcribe" );
  
  actOptions = new KAction(i18n("&Configure KTT..."), "configure", 0, this, SLOT(slotOptions()), actionCollection(), "settings_options" );
  actLoadMode = new KAction(i18n("Load &Mode..."), "fileimport", CTRL+Key_M, this, SLOT(slotLoadMode()), actionCollection(), "load_mode" );
  
  actWebHome = new KAction(i18n("KTT Homepage"), SmallIconSet("www"), 0, this, SLOT(slotWebHome()), actionCollection(), "help_web_home");
  actWebYaTT = new KAction(i18n("YaTT (Narmacil) Homepage"), SmallIconSet("www"), 0, this, SLOT(slotWebYaTT()), actionCollection(), "help_web_yatt");

  initWhatsThis();  // add "What's This" descriptions to the actions
}
                    
void KTTApp::initStatusBar()
{
  statusBar()->insertItem(i18n("Ready."), ID_STATUS_MSG, 1);
  statusBar()->insertItem("", ID_STATUS_MODE_FILE );
  statusBar()->insertItem("", ID_STATUS_MODE_CASE );
  statusBar()->insertItem("", ID_STATUS_MODE_COMMENT );
  
  statusBar()->setItemAlignment(ID_STATUS_MSG, AlignLeft);  
}

void KTTApp::setDualCaption()
{
  if ( configStruct.fullPathCaption )
    setCaption( doc->URL(DOCUMENT_TYPE_ROMAN).prettyURL(0, KURL::StripFileProtocol) + " | " +
                doc->URL(DOCUMENT_TYPE_TENGWAR).prettyURL(0, KURL::StripFileProtocol), false );
  else
    setCaption( doc->URL(DOCUMENT_TYPE_ROMAN).fileName() + " | " +
                doc->URL(DOCUMENT_TYPE_TENGWAR).fileName(), false );
}

void KTTApp::openDocumentFile(KURL& url, const int type)
{
  QString msg;
  slotStatusMsg(i18n("Opening file..."));

  if ( !doc->openDocument(url, type) )
    msg = i18n("Error opening %1").arg(url.prettyURL());
  url.setProtocol( ((type == DOCUMENT_TYPE_TENGWAR)?PROTOCOL_TENGWAR:PROTOCOL_ROMAN) + url.protocol() );
  actFileOpenRecent->addURL(url);
  setDualCaption();  

  slotStatusMsg(i18n("Ready."));
  if ( !msg.isEmpty() ) slotStatusHelpMsg(msg);
}
                               
KTTDoc *KTTApp::getDocument() const
{
  return doc;
}

void KTTApp::saveOptions()
{
  config->setGroup("General Options");
  
  config->writeEntry("Window Size", size());
  config->writeEntry("Window Position", pos());  
  config->writeEntry("Show Main Toolbar", actViewToolBarMain->isChecked());
  config->writeEntry("Show Actions Toolbar", actViewToolBarActions->isChecked());
  config->writeEntry("Show Statusbar", actViewStatusBar->isChecked());
  config->writeEntry("Main Toolbar Position", (int) toolBar("mainToolBar")->barPos());
  config->writeEntry("Actions Toolbar Position", (int) toolBar("actionsToolBar")->barPos());
  actFileOpenRecent->saveEntries(config,"Recent Files");
  config->writeEntry("Font Tengwar", configStruct.fontTengwar);
  config->writeEntry("Font Roman", configStruct.fontRoman);
  config->writeEntry("Auto Transcribe", configStruct.autoTranscribe);
  config->writeEntry("Clear Target", configStruct.clearTarget);
  config->writeEntry("Full Caption", configStruct.fullPathCaption);
  config->writeEntry("Show Transcription Time", configStruct.showTranscriptionTime);
  switch (configStruct.onStartup)
  {
    case STARTUP_NONE:   config->writeEntry("Startup", "none");   break;
    case STARTUP_RELOAD: config->writeEntry("Startup", "reload"); break;
    case STARTUP_LOAD:   config->writeEntry("Startup", "load");   break;
  }
  config->writeEntry("Startup Mode", configStruct.startupMode);
  config->writeEntry("Recent Mode", configStruct.currentMode);
  config->writeEntry("Tengwar Fonts", configStruct.tengwarFonts);
  switch (configStruct.listItemMarker)
  {
    case NUMBER_MARKER_NONE: config->writeEntry("ListItem Marker", "none"); break;
    case NUMBER_MARKER_DOT:  config->writeEntry("ListItem Marker", "dot");  break;
    case NUMBER_MARKER_BAR:  config->writeEntry("ListItem Marker", "bar");  break;
  }
                               
  config->setGroup("Narmacil Options");
  
  config->writeEntry("Errors", configStruct.narmacil.alert ? "alert" : "continue");
  config->writeEntry("Smart", configStruct.narmacil.smart);
  config->writeEntry("Mark Digits", configStruct.narmacil.digits);
  config->writeEntry("Mark LSD", configStruct.narmacil.LSD);
  config->writeEntry("Numeric Base", configStruct.narmacil.decimal ? "10" : "12");
  config->writeEntry("Treat Numbers Separately", configStruct.narmacil.separateNums);
}                        

void KTTApp::readOptions()
{
  config->setGroup("General Options");

  // restore toolbars + position
  actViewToolBarMain->setChecked(config->readBoolEntry("Show Main Toolbar", true));
  slotViewToolBar(actViewToolBarMain, "main");
  toolBar("mainToolBar")->setBarPos((KToolBar::BarPosition) config->readNumEntry("Main Toolbar Position", KToolBar::Top));
  actViewToolBarActions->setChecked(config->readBoolEntry("Show Actions Toolbar", true));
  slotViewToolBar(actViewToolBarActions, "actions");
  toolBar("actionsToolBar")->setBarPos((KToolBar::BarPosition) config->readNumEntry("Actions Toolbar Position", KToolBar::Top));

  actViewStatusBar->setChecked(config->readBoolEntry("Show Statusbar", true));
  slotViewStatusBar();
  
  // initialize the recent file list
  actFileOpenRecent->loadEntries(config, "Recent Files");

  // restore window size + position
  QSize size=config->readSizeEntry("Window Size");
  if (!size.isEmpty()) resize(size);
  QSize pos=config->readSizeEntry("Window Position");
  if (!pos.isEmpty()) move(pos.width(), pos.height());  

  // ktt general options
  configStruct.fontRoman = config->readFontEntry("Font Roman");
  if (!configStruct.fontRoman.family().isEmpty()) view->setFontRoman(configStruct.fontRoman);
  configStruct.fontTengwar = config->readFontEntry("Font Tengwar");
  if (!configStruct.fontTengwar.family().isEmpty()) view->setFontTengwar(configStruct.fontTengwar);

  actAutoTranscribe->setChecked(config->readBoolEntry("Auto Transcribe", false));
  slotTransAvailable();
  configStruct.clearTarget = config->readBoolEntry("Clear Target", true);
  configStruct.fullPathCaption = config->readBoolEntry("Full Caption", false);
  configStruct.showTranscriptionTime = config->readBoolEntry("Show Transcription Time", true);
  QString s = config->readEntry("Startup", "none");
  if (s == "reload")
    configStruct.onStartup = STARTUP_RELOAD;
  else if (s == "load")
    configStruct.onStartup = STARTUP_LOAD;
  else
    configStruct.onStartup = STARTUP_NONE;
  configStruct.recentMode = config->readEntry("Recent Mode", "");
  configStruct.startupMode = config->readEntry("Startup Mode", "");
  configStruct.tengwarFonts = config->readListEntry("Tengwar Fonts");
  if ( configStruct.tengwarFonts.isEmpty() )
  {
    configStruct.tengwarFonts += "Tengwar Quenya";
    configStruct.tengwarFonts += "Tengwar Sindarin";
    configStruct.tengwarFonts += "Tengwar Noldor";
    configStruct.tengwarFonts += "Tengwar Parmaite";
    configStruct.tengwarFonts += "Tengwar Cursive";
  }
  s = config->readEntry("ListItem Marker", "dot");
  if ( s == "dot" )
    configStruct.listItemMarker = NUMBER_MARKER_DOT;
  else if ( s == "bar" )
    configStruct.listItemMarker = NUMBER_MARKER_BAR;
  else
    configStruct.listItemMarker = NUMBER_MARKER_NONE;
 
  
  config->setGroup("Narmacil Options");

  configStruct.narmacil.alert = (config->readEntry("Errors", "continue") == "alert");
  configStruct.narmacil.smart = config->readBoolEntry("Smart", true);
  configStruct.narmacil.digits = config->readBoolEntry("Mark Digits", false);
  configStruct.narmacil.LSD = config->readBoolEntry("Mark LSD", false);
  configStruct.narmacil.decimal = (config->readEntry("Numeric Base", "12") == "10");
  configStruct.narmacil.separateNums = config->readBoolEntry("Treat Numbers Separately", true);
  
  switch (configStruct.onStartup)
  {
    case STARTUP_RELOAD: loadNarmacilMode(configStruct.recentMode); break;
    case STARTUP_LOAD:   loadNarmacilMode(configStruct.startupMode); break;
  }
}

void KTTApp::saveProperties(KConfig *_cfg)
{
  if ( doc->URL(DOCUMENT_TYPE_ROMAN).fileName() != i18n("Untitled") && !doc->isModifiedRoman() )
  {
    // saving to tempfile not necessary
  }
  else
  {
    KURL url = doc->URL(DOCUMENT_TYPE_ROMAN);
    _cfg->writeEntry("filenameRoman", url.url());
    _cfg->writeEntry("modifiedRoman", doc->isModifiedRoman());
    QString tempname = kapp->tempSaveName(url.url());
    QString tempurl= KURL::encode_string(tempname);
    doc->saveDocument(KURL(tempurl), DOCUMENT_TYPE_ROMAN);
  }

  if ( doc->URL(DOCUMENT_TYPE_TENGWAR).fileName() != i18n("Untitled") && !doc->isModifiedTengwar() )
  {
    // saving to tempfile not necessary
  }
  else
  {
    KURL url = doc->URL(DOCUMENT_TYPE_TENGWAR);
    _cfg->writeEntry("filenameTengwar", url.url());
    _cfg->writeEntry("modifiedTengwar", doc->isModifiedTengwar());
    QString tempname = kapp->tempSaveName(url.url());
    QString tempurl= KURL::encode_string(tempname);
    doc->saveDocument(KURL(tempurl), DOCUMENT_TYPE_TENGWAR);
  }
}                         

void KTTApp::readProperties(KConfig* _cfg)
{
  QString filename = _cfg->readEntry("filenameRoman", "");
  bool modified = _cfg->readBoolEntry("modifiedRoman", false);
  if ( modified )
  {
    bool canRecover;
    QString tempname = kapp->checkRecoverFile(filename, canRecover);
    KURL url(tempname);
    if ( canRecover )
    {
      doc->openDocument(url, DOCUMENT_TYPE_ROMAN);
      doc->setModifiedRoman(modified);
      QFile::remove(tempname);
    }
  }
  else
  {
    if ( !filename.isEmpty() )
    {
      KURL url(filename);
      doc->openDocument(url, DOCUMENT_TYPE_ROMAN);
    }
  }

  filename = _cfg->readEntry("filenameTengwar", "");
  modified = _cfg->readBoolEntry("modifiedTengwar", false);
  if ( modified )
  {
    bool canRecover;
    QString tempname = kapp->checkRecoverFile(filename, canRecover);
    KURL url(tempname);
    if ( canRecover )
    {
      doc->openDocument(url, DOCUMENT_TYPE_TENGWAR);
      doc->setModifiedTengwar(modified);
      QFile::remove(tempname);
    }
  }
  else
  {
    if ( !filename.isEmpty() )
    {
      KURL url(filename);
      doc->openDocument(url, DOCUMENT_TYPE_TENGWAR);
    }
  }
  
  setDualCaption();
}

bool KTTApp::queryClose()
{
  return doc->saveModified();
}

bool KTTApp::queryExit()
{
  saveOptions();
  return true;
}

void KTTApp::loadNarmacilMode(const KURL& url)
{
  if (!url.isEmpty())
  {
    QString tmpfile;
    if ( KIO::NetAccess::download(url, tmpfile) )
    {
      QFile f(tmpfile);
      try
      {
        if ( narmacil ) delete narmacil;
        narmacil = new CTranscription;

        narmacil->LoadMode(tmpfile);
        narmacil->SetAlert(configStruct.narmacil.alert);
        narmacil->SetSmart(configStruct.narmacil.smart);
        narmacil->SetDigits(configStruct.narmacil.digits);
        narmacil->SetLSD(configStruct.narmacil.LSD);
        narmacil->SetDecimal(configStruct.narmacil.decimal);
        narmacil->SetSeparateNums(configStruct.narmacil.separateNums);

        configStruct.currentMode = url.url();
        statusBar()->changeItem(url.fileName(), ID_STATUS_MODE_FILE);
        statusBar()->changeItem(narmacil->GetModeCaseSensitivity() ? i18n("Case sensitive") : i18n("Case insensitive"), ID_STATUS_MODE_CASE);
        statusBar()->changeItem(narmacil->GetModeComment(), ID_STATUS_MODE_COMMENT);
        slotStatusMsg(i18n("Ready."));
        slotStatusHelpMsg(i18n("%1 loaded.").arg(url.fileName()));
        slotTransAvailable();
      }
      catch(exception &e)
      {
        fprintf(stderr, "%s\n", e.what());
        configStruct.currentMode = "";
        statusBar()->changeItem("", ID_STATUS_MODE_FILE);
        statusBar()->changeItem("", ID_STATUS_MODE_CASE);
        statusBar()->changeItem("", ID_STATUS_MODE_COMMENT);
        slotStatusMsg(i18n("Error opening %1 - %2").arg(url.fileName()).arg(i18n(e.what())));
        slotTransAvailable();
      }
    }
    KIO::NetAccess::removeTempFile(tmpfile);
  }
}

void KTTApp::fileOpen(const int type)
{
  QString msg;
  slotStatusMsg(i18n("Opening file..."));

  if ( !doc->saveModified() )
  {
     // here saving wasn't successful
  }
  else
  {
    QString fileMaskFilter = "";
    switch (type)
    {
      case DOCUMENT_TYPE_ROMAN:
//        fileMaskFilter = i18n("*.txt *.rom|Roman files\n");
        break;      
      case DOCUMENT_TYPE_TENGWAR:
        fileMaskFilter = i18n("*.tng|Tengwar files\n");
        break;
    }                        
    fileMaskFilter = fileMaskFilter + i18n("*|All files");
    KURL url = KFileDialog::getOpenURL(QString::null, fileMaskFilter, this,
      (type == DOCUMENT_TYPE_TENGWAR)?i18n("Open Tengwar File..."):i18n("Open Roman File..."));
    if ( !url.isEmpty() )
    {
      if ( !doc->openDocument(url, type) )
        msg = i18n("Error opening %1").arg(url.prettyURL());
      url.setProtocol( ((type == DOCUMENT_TYPE_TENGWAR)?PROTOCOL_TENGWAR:PROTOCOL_ROMAN) + url.protocol() );
      actFileOpenRecent->addURL( url );
      setDualCaption();
    }
    slotSaveAvailable();
  }

  slotStatusMsg(i18n("Ready."));
  if ( !msg.isEmpty() ) slotStatusHelpMsg(msg);
}                   

void KTTApp::fileSaveAs(const int type)
{
  QString msg;
  QString fileMaskFilter = "";

  switch (type)
  {
    case DOCUMENT_TYPE_ROMAN:
      slotStatusMsg(i18n("Saving Roman file with a new filename..."));
//      fileMaskFilter = i18n("*.txt *.rom|Roman files\n");
      break;
    case DOCUMENT_TYPE_TENGWAR:
      slotStatusMsg(i18n("Saving Tengwar file with a new filename..."));
      fileMaskFilter = i18n("*.tng|Tengwar files\n");
      break;
  }
  fileMaskFilter = fileMaskFilter + i18n("*|All files");
  KURL url = KFileDialog::getSaveURL(QDir::currentDirPath(),
        fileMaskFilter, this,
        (type == DOCUMENT_TYPE_TENGWAR)?i18n("Save Tengwar file as..."):i18n("Save Roman file as...") );
  if ( !url.isEmpty() )
  {
    switch (type)
    {
      case DOCUMENT_TYPE_ROMAN:
//        if ( !(url.path(-1).endsWith(".txt") || url.path(-1).endsWith(".rom")) )
//          url.setPath(url.path(-1) + ".txt");
        break;
      case DOCUMENT_TYPE_TENGWAR:
        if ( !(url.path(-1).endsWith(".tng")) )
          url.setPath(url.path(-1) + ".tng");
        break;
    }
KMessageBox::information( this, url.url() );
        
    if ( !doc->saveDocument(url, type) )
      msg = i18n("Error saving %1").arg(url.prettyURL());
    url.setProtocol( ((type == DOCUMENT_TYPE_TENGWAR)?PROTOCOL_TENGWAR:PROTOCOL_ROMAN) + url.protocol() );
    actFileOpenRecent->addURL(url);
    setDualCaption();
  }

  slotSaveAvailable();
  slotStatusMsg(i18n("Ready."));
  if ( !msg.isEmpty() ) slotStatusHelpMsg(msg);
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

void KTTApp::slotViewToolBar(KToggleAction* &act, const QString &name)
{
  // toolbar name in kttui.rc = <name>ToolBar
  // action name = show_toolbar_<name>

  slotStatusMsg(i18n("Toggling toolbar %1...").arg(name+"ToolBar"));
  if ( act->isChecked() )
    toolBar(name+"ToolBar")->show();
  else
    toolBar(name+"ToolBar")->hide();
  slotStatusMsg(i18n("Ready."));
}

void KTTApp::slotViewStatusBar()
{
  slotStatusMsg(i18n("Toggle the statusbar..."));
  if (!actViewStatusBar->isChecked())
    statusBar()->hide();
  else
    statusBar()->show();
  slotStatusMsg(i18n("Ready."));
}

void KTTApp::slotStatusMsg(const QString &text)
{
  // change status message permanently
  statusBar()->clear();
  statusBar()->changeItem(text, ID_STATUS_MSG);
  statusBar()->repaint();
}

void KTTApp::slotStatusHelpMsg(const QString& text)
{
  // change status message of whole statusbar temporary (text, msec)
  statusBar()->message(text, 2000);
}

void KTTApp::slotWebHome()
{
  slotStatusMsg(i18n("Opening web site..."));
  kapp->invokeBrowser( WWW_KTT );
  slotStatusMsg(i18n("Ready."));
}

void KTTApp::slotWebYaTT()
{
  slotStatusMsg(i18n("Opening web site..."));
  kapp->invokeBrowser( WWW_NARMACIL );
  slotStatusMsg(i18n("Ready."));
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION - FILES
/////////////////////////////////////////////////////////////////////

void KTTApp::slotFileNew()
{
  slotStatusMsg(i18n("Creating new document..."));

  if ( !doc->saveModified() )
  {
     // here saving wasn't successful
  }
  else
  {
    doc->newDocument();
    setDualCaption();
    slotSaveAvailable();
  }
  slotStatusMsg(i18n("Ready."));
}

void KTTApp::slotFileOpenRoman()
{
  fileOpen(DOCUMENT_TYPE_ROMAN);
}

void KTTApp::slotFileOpenTengwar()
{
  fileOpen(DOCUMENT_TYPE_TENGWAR);
}

void KTTApp::slotFileOpenRecent(const KURL& _url)
{
  QString msg;
  slotStatusMsg(i18n("Opening file..."));

  if ( !doc->saveModified() )
  {
     // here saving wasn't successful
  }
  else
  {
    QString pu = _url.prettyURL();
    KURL url;
    int type = 0;

    if ( pu.left(PROTOCOL_TENGWAR.length()) == PROTOCOL_TENGWAR )
    {
      url = pu.right(pu.length() - PROTOCOL_TENGWAR.length());
      type = DOCUMENT_TYPE_TENGWAR;
    }
    else if ( pu.left(PROTOCOL_ROMAN.length()) == PROTOCOL_ROMAN ) {
      url = pu.right(pu.length() - PROTOCOL_ROMAN.length());
      type = DOCUMENT_TYPE_ROMAN;
    }    

    if ( !doc->openDocument(url, type) )
      msg = i18n("Error opening %1").arg(url.prettyURL());
    setDualCaption();
    slotSaveAvailable();
  }

  slotStatusMsg(i18n("Ready."));
  if ( !msg.isEmpty() ) slotStatusHelpMsg(msg);
}

void KTTApp::slotFileSave()
{
  slotStatusMsg(i18n("Saving file..."));

  KURL url = doc->URL(view->currentType());
  if ( url.fileName() == i18n("Untitled") )
    fileSaveAs(view->currentType());
  else
    doc->saveDocument(url, view->currentType());

  slotSaveAvailable();            
  slotStatusMsg(i18n("Ready."));
}

void KTTApp::slotFileSaveAs()
{
  fileSaveAs(view->currentType());
}

void KTTApp::slotFileSaveAsRoman()
{
  fileSaveAs(DOCUMENT_TYPE_ROMAN);
}

void KTTApp::slotFileSaveAsTengwar()
{
  fileSaveAs(DOCUMENT_TYPE_TENGWAR);
}

void KTTApp::slotFileClose()
{
  close();
}

void KTTApp::slotFilePrint()
{
  slotStatusMsg((view->currentType()==DOCUMENT_TYPE_ROMAN)?i18n("Printing Roman file..."):i18n("Printing Tengwar file..."));

  QPrinter printer;
  if ( printer.setup(this) )
    view->print(&printer);

  slotStatusMsg(i18n("Ready."));
}

void KTTApp::slotFileQuit()
{
  this->close();
/*
  slotStatusMsg(i18n("Exiting..."));

  // close the first window, the list makes the next one the first again.
  // This ensures that queryClose() is called on each window to ask for closing
  KMainWindow* w;
  if(memberList)
  {
    for(w=memberList->first(); w!=0; w=memberList->first())
    {
      // only close the window if the closeEvent is accepted.
      // If the user presses Cancel on the saveModified() dialog,
      // the window and the application stay open.
      if(!w->close())
        break;
    }
  }
*/  
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION - EDITING
/////////////////////////////////////////////////////////////////////

void KTTApp::slotSaveAvailable()
{
  actFileSave->setEnabled(doc->isModifiedRoman() || doc->isModifiedTengwar());
}  
    
void KTTApp::slotCopyAvailable(bool yes)
{
  actEditCopy->setEnabled(yes);
  actEditCut->setEnabled(yes);
}

void KTTApp::slotEditCut()
{
  slotStatusMsg(i18n("Cutting selection..."));
  view->cut();
  actEditPaste->setEnabled(true);  
  slotStatusMsg(i18n("Ready."));
}

void KTTApp::slotEditCopy()
{
  slotStatusMsg(i18n("Copying selection to clipboard..."));
  view->copy();
  actEditPaste->setEnabled(true);
  slotStatusMsg(i18n("Ready."));
}

void KTTApp::slotEditPaste()
{
  slotStatusMsg(i18n("Inserting clipboard contents..."));
  view->paste();
  slotStatusMsg(i18n("Ready."));
}

void KTTApp::slotInsertNumber()
{
  slotStatusMsg(i18n("Insert number..."));

  NumberForm *dlg = new NumberForm(this, "insert number", true);

  dlg->exec();
  delete dlg;
  
  slotStatusMsg(i18n("Ready."));
}

void KTTApp::slotInsertListItem()
{
  slotStatusMsg(i18n("Insert list item..."));

  ListItemForm *dlg = new ListItemForm(this, "insert list item", true);
  dlg->tlItemSample->setFont(configStruct.fontTengwar);
  switch (configStruct.listItemMarker)
  {
    case NUMBER_MARKER_NONE: dlg->rbMarkNone->setChecked(true); break;
    case NUMBER_MARKER_DOT:  dlg->rbMarkDot->setChecked(true);  break;
    case NUMBER_MARKER_BAR:  dlg->rbMarkBar->setChecked(true);  break;
  }
  if ( dlg->exec() )
  {
    view->setTengwar(dlg->tlItemSample->text(), false);  
  }
  if ( dlg->rbMarkNone->isChecked() )
    configStruct.listItemMarker = NUMBER_MARKER_NONE;
  else if ( dlg->rbMarkBar->isChecked() )
    configStruct.listItemMarker = NUMBER_MARKER_BAR;
  else
    configStruct.listItemMarker = NUMBER_MARKER_DOT;

  delete dlg;

  slotStatusMsg(i18n("Ready."));
}

  
/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION - CONFIGURATION
/////////////////////////////////////////////////////////////////////

void KTTApp::slotOptions()
{
  KTTSetupDlg *dlg = new KTTSetupDlg(this);
  dlg->setOptions();    // set current values

  if ( dlg->exec() )
  {
    dlg->getOptions();  // get new settings
    slotTransAvailable();
    setDualCaption();
            
    saveOptions();
    slotStatusHelpMsg(i18n("Configuration saved."));
  }
  delete dlg;
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION - TRANSCRIPTION
/////////////////////////////////////////////////////////////////////

void KTTApp::slotLoadMode()
{
  slotStatusMsg(i18n("Opening mode file..."));
  KURL url = KFileDialog::getOpenURL(QString::null,
      i18n("*.mod *.tmf|All mode files\n"
           "*.mod|Tengwar Scribe mode files\n"
           "*.tmf|YaTT Tengwar Mode Files\n"
           "*|All files"), this, i18n("Open Mode File..."));
  if ( !url.isEmpty() )
    loadNarmacilMode(url);
  else
    slotStatusMsg(i18n("Ready."));
}

void KTTApp::slotTransAvailable()
{
  if ( configStruct.currentMode.isEmpty() )
  {
    actTransToTengwar->setEnabled(false);
    actTransToRoman->setEnabled(false);
  }
  else
  {
    actTransToTengwar->setEnabled(!actAutoTranscribe->isChecked());
    actTransToRoman->setEnabled(!actAutoTranscribe->isChecked());
  }
  configStruct.autoTranscribe = actAutoTranscribe->isChecked();
}

void KTTApp::slotRomanTextChanged()
{
  slotSaveAvailable();
  if ( configStruct.autoTranscribe && view->hasFocusRoman() )
    slotTransToTengwar();
}

void KTTApp::slotTengwarTextChanged()
{
  slotSaveAvailable();
  if ( configStruct.autoTranscribe && view->hasFocusTengwar() )
    slotTransToRoman();
}

void KTTApp::slotTransToTengwar()
{
  if ( !narmacil || configStruct.currentMode.isEmpty() ) return;

  QString msg;
  if ( !configStruct.autoTranscribe )
    slotStatusMsg(i18n("Transcribing..."));
  try
  {
    bool showTime = configStruct.showTranscriptionTime && !configStruct.autoTranscribe;
    QTime t;
    if ( showTime ) t.start();
    
    view->setTengwar(
      narmacil->Roman2Tengwar( KTTCodec::toWindows( view->textRoman(), narmacil->GetModeCaseSensitivity() ) ),
      configStruct.clearTarget || configStruct.autoTranscribe );

    if ( showTime )
    {
      QString ts;
      ts.sprintf("%.3f", ((double)t.elapsed() / 1000));  
      msg = i18n("Transcription done in %1 sec.").arg(ts);
    }
  }
  catch (EPatternNotFound &e)
  {
    AlertForm *dlg = new AlertForm(this, "alert", true);
    dlg->tlWhat->setFont(configStruct.fontRoman);
    dlg->tlWhat->setText(KTTCodec::fromWindows(e.what(), true));
    dlg->exec();
    delete dlg;
  }
  catch (EAbort &e)
  {
    KMessageBox::error(this, e.what(), i18n("Transcription aborted"));
  }
  slotStatusMsg(i18n("Ready."));
  if ( !msg.isEmpty() ) slotStatusHelpMsg(msg);
}

void KTTApp::slotTransToRoman()
{
  if ( !narmacil || configStruct.currentMode.isEmpty() ) return;

  QString msg;
  if ( !configStruct.autoTranscribe )
    slotStatusMsg(i18n("Transcribing..."));
  try
  {
    bool showTime = configStruct.showTranscriptionTime && !configStruct.autoTranscribe;
    QTime t;
    if ( showTime ) t.start();

    view->setRoman(
      KTTCodec::fromWindows( narmacil->Tengwar2Roman(view->textTengwar()), narmacil->GetModeCaseSensitivity() ),
      configStruct.clearTarget || configStruct.autoTranscribe );

    if ( showTime )
    {
      QString ts;
      ts.sprintf("%.3f", ((double)t.elapsed() / 1000));
      msg = i18n("Transcription done in %1 sec.").arg(ts);
    }
  }
  catch (EPatternNotFound &e)
  {
    AlertForm *dlg = new AlertForm(this, "alert", true);
    dlg->tlWhat->setFont(configStruct.fontTengwar);
    dlg->tlWhat->setText(e.what());
    QToolTip::add(dlg->tlWhat, e.what());
    dlg->exec();
    delete dlg;
  }
  catch (EAbort &e)
  {
    KMessageBox::error(this, e.what(), i18n("Transcription aborted"));
  }
  slotStatusMsg(i18n("Ready."));
  if ( !msg.isEmpty() ) slotStatusHelpMsg(msg);
}

void KTTApp::slotToggleTengwarFont()
{
  slotStatusMsg(i18n("Changing target font..."));

  if ( actUseTengwarFont->isChecked() )
    view->setFontTengwar(configStruct.fontTengwar);
  else
    view->setFontTengwar(configStruct.fontRoman);
  
  slotStatusMsg(i18n("Ready."));
}


