/***************************************************************************
   kttdoc.cpp  -  KTTDoc

   begin     : 2002.11.08
   copyright : (C) 2002 by MiMaS
   email     : mimas@mimas.ceti.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// include files for Qt
#include <qdir.h>
#include <qwidget.h>
#include <qtextedit.h>

// include files for KDE
#include <klocale.h>
#include <kmessagebox.h>
#include <kio/job.h>
#include <kio/netaccess.h>
#include <ktempfile.h>

// application specific includes
#include "kttdoc.h"
#include "ktt.h"


KTTDoc::KTTDoc(QWidget *parent, const char *name) : QObject(parent, name)
{
}

KTTDoc::~KTTDoc()
{
}

void KTTDoc::addView(KTTView *view)
{
  pView = view;
}
/*
void KTTDoc::setURL(const KURL &url)
{
  doc_url = url;
}
*/
const KURL& KTTDoc::URL(const int type) const
{
  switch (type)
  {
    case DOCUMENT_TYPE_ROMAN:   return doc_url_roman;
    case DOCUMENT_TYPE_TENGWAR: return doc_url_tengwar;
  }
  return doc_url_roman; // not gonna happen
}

void KTTDoc::slotUpdateAllViews(KTTView *sender)
{
  pView->repaint();
}

bool KTTDoc::saveModified()
{
  bool completedRoman   = saveModifiedType(DOCUMENT_TYPE_ROMAN);
  bool completedTengwar = saveModifiedType(DOCUMENT_TYPE_TENGWAR);
  return completedRoman && completedTengwar;
}

bool KTTDoc::saveModifiedType(int type)
{
  bool completed = true;        
  bool modified = false;
  switch (type)
  {
    case DOCUMENT_TYPE_ROMAN:
      modified = pView->isModifiedRoman();
      break;
    case DOCUMENT_TYPE_TENGWAR:
      modified = pView->isModifiedTengwar();
      break;
  } 
  if (modified)
  {
    KTTApp *win = (KTTApp *)parent();
    int want_save = KMessageBox::warningYesNoCancel(win,
           (type == DOCUMENT_TYPE_TENGWAR?i18n("The current tengwar file has been modified."):i18n("The current roman file has been modified."))
           + "\n" + i18n("Do you want to save it?"), i18n("Warning"));
    switch (want_save)
    {
      case KMessageBox::Yes:
           switch (type)
           {
             case DOCUMENT_TYPE_ROMAN:
               if (doc_url_roman.fileName() == i18n("Untitled"))
                 win->slotFileSaveAsRoman();
               else
                 saveDocument(URL(DOCUMENT_TYPE_ROMAN), DOCUMENT_TYPE_ROMAN);
               break;
             case DOCUMENT_TYPE_TENGWAR:
               if (doc_url_tengwar.fileName() == i18n("Untitled"))
                 win->slotFileSaveAsTengwar();
               else
                 saveDocument(URL(DOCUMENT_TYPE_TENGWAR), DOCUMENT_TYPE_TENGWAR);
               break;
           }
           completed = true;
           break;
      case KMessageBox::No:
           switch (type)
           {
             case DOCUMENT_TYPE_ROMAN:
               pView->setModifiedRoman(false);
               break;
             case DOCUMENT_TYPE_TENGWAR:
               pView->setModifiedTengwar(false);
               break;
           }
           completed = true;
           break;
      case KMessageBox::Cancel:
      default:
           completed = false;
           break;
    }
  }
  return completed;
}

void KTTDoc::closeDocument()
{
}

bool KTTDoc::newDocument()
{
  pView->clearBoth();   // no document data in KTTDoc, so just clear both panes in KTTView
  pView->setModifiedRoman(false);
  pView->setModifiedTengwar(false);
  doc_url_roman.setFileName(i18n("Untitled"));
  doc_url_tengwar.setFileName(i18n("Untitled"));
  return true;
}

bool KTTDoc::openDocument(const KURL& url, const int type)
{
  bool result = false;
  QString tmpfile;
  if ( KIO::NetAccess::download(url, tmpfile) )
  {
    QFile f(tmpfile);
    if ( f.open(IO_ReadOnly) )
    {
      Q_ULONG len = f.size();
      char *bufor = (char *)malloc((len+1)*sizeof(char));
      f.readBlock(bufor, len);
      bufor[len] = 0;

      // actually document text is in one of KTTView panes, not in KTTDoc
      switch (type)
      {
        case DOCUMENT_TYPE_ROMAN:
          pView->setRoman(bufor);
          pView->setModifiedRoman(false);
          doc_url_roman = url;
          result = true;
          break;
        case DOCUMENT_TYPE_TENGWAR:
          pView->setTengwar(bufor);
          pView->setModifiedTengwar(false);
          doc_url_tengwar = url;
          result = true;
          break;
      }

      free(bufor);
      f.close();
    }
    KIO::NetAccess::removeTempFile(tmpfile);

    pView->rewindBoth();
  }
  return result;  
}

bool KTTDoc::saveDocument(const KURL& url, const int type)
{
  QString text;
  bool modified = false;
  switch (type)
  {
    case DOCUMENT_TYPE_ROMAN:
      text = pView->textRoman(false);
      modified = pView->isModifiedRoman();
      break;
    case DOCUMENT_TYPE_TENGWAR:
      text = pView->textTengwar(false);
      modified = pView->isModifiedTengwar();
      break;
  }
  if ( !text.isEmpty() && modified )
  {
    KTempFile tmp;
    tmp.file()->writeBlock(text.data(), text.length());
    tmp.file()->flush();
    tmp.file()->close();
    KIO::NetAccess::upload(tmp.name(), url);
    tmp.unlink();
  }      
  switch (type)
  {
    case DOCUMENT_TYPE_ROMAN:
      doc_url_roman = url;
      pView->setModifiedRoman(false);
      break;
    case DOCUMENT_TYPE_TENGWAR:
      doc_url_tengwar = url;
      pView->setModifiedTengwar(false);
      break;
  }
  return true;
}

