/***************************************************************************
   kttview.h  -  KTTView

   begin     : 2002.11.08
   copyright : (C) 2002 by MiMaS
   email     : mimas@mimas.ceti.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KTTVIEW_H
#define KTTVIEW_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

// include files for Qt
#include <qwidget.h>                           
#include <qstring.h>
#include <qtextedit.h>
#include <qvariant.h>
#include <qwidget.h>
class QVBoxLayout;
class QHBoxLayout;
class QGridLayout;
class QSplitter;
//class QTextEdit;

// include files fot KDE
#include <kled.h>

#include "sigqtextedit.h"


class KTTDoc;

class KTTView : public QWidget
{
  Q_OBJECT
  public:
    KTTView(QWidget *parent = 0);
    ~KTTView();

    KTTDoc *getDocument() const;
                             
    /** contains the implementation for printing functionality */
    void print(QPrinter *pPrinter);

    void clearTengwar();
    void clearRoman();
    void clearBoth() { clearRoman(); clearTengwar(); };

    void rewindBoth();

    void setTengwar(const QString & text, bool clear = true);
    void setRoman(const QString & text, bool clear = true);
    QString textTengwar(bool selectedOnly = true) const;
    QString textRoman(bool selectedOnly = true) const;

    void setFontTengwar(const QFont & font);
    void setFontRoman(const QFont & font);
    QFont getFontTengwar() const { return fontTengwar; };
    QFont getFontRoman() const { return fontRoman; };

    void setModifiedRoman(bool yes)   { TextEditLeft->setModified(yes);  };
    void setModifiedTengwar(bool yes) { TextEditRight->setModified(yes); };
    bool isModifiedRoman()   const { return TextEditLeft->isModified();  };
    bool isModifiedTengwar() const { return TextEditRight->isModified(); };

    bool hasFocusRoman() const { return TextEditLeft->hasFocus(); };
    bool hasFocusTengwar() const { return TextEditRight->hasFocus(); };
    int currentType() const { return currentDocumentType; };
                                   
  public slots:
    void cut();
    void copy();
    void paste();
    void slotTextEditFocusChanged();
    void slotTextEditModificationChanged(bool);
                            
  private:
    QFont fontTengwar;
    QFont fontRoman;
    int currentDocumentType;

  protected:
    QGridLayout* MainFormLayout;
    QVBoxLayout* LayoutLeft;     
    QHBoxLayout* LayoutLeftDown; 
    QVBoxLayout* LayoutRight;    
    QHBoxLayout* LayoutRightDown;
    QSplitter* Splitter;
    KLed* LedLeft;                // panel focus LED
    KLed* LedRight;
    KLed* LedModLeft;             // panel modification
    KLed* LedModRight;

    sigQTextEdit* TextEditLeft;
    sigQTextEdit* TextEditRight;
};

#endif // KTTVIEW_H
