/***************************************************************************
   main.cpp  -  KTT : KDE Tengwar Tool

   begin     : 2002.11.08
   copyright : (C) 2002 by MiMaS
   email     : mimas@mimas.ceti.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kcmdlineargs.h>
#include <kaboutdata.h>
#include <klocale.h>

#include "resources.h"
#include "ktt.h"
#include "splashscreen.h"
#include "narmacil/transcription.h"


#define KTTVER "1.1.1"


#undef VERSION    // defined in config.h
#ifdef ENGINEVER  // defined in narmacil/transcription.h
#define VERSION KTTVER" ["ENGINEVER"]"
#else
#define VERSION KTTVER
#endif

static const char *description =
  I18N_NOOP("KTT : KDE Tengwar Tool\n"
            "powered by Narmacil transcription engine");

static KCmdLineOptions options[] =
{
  { "nosplash", I18N_NOOP("Do not show splashscreen on startup"), 0 },  // --splash / --nosplash
  { "r", 0, 0 },                                                        // -r is an alias for --roman
  { "roman <file>", I18N_NOOP("Open Roman file"), 0 },                  // --roman option takes an argument (URL)
  { "t", 0, 0 },                                                        // -t is an alias for --tengwar
  { "tengwar <file>", I18N_NOOP("Open Tengwar file"), 0 },              // --tengwar option takes an argument (URL)
  { "m", 0, 0 },                                                        // -m is an alias for --mode
  { "mode <file>", I18N_NOOP("Load mode file"), 0 },                    // --mode option takes an argument (URL)
  { 0, 0, 0 }
};

int main(int argc, char *argv[])
{
  KAboutData aboutData("ktt", "KTT",
    VERSION, description, KAboutData::License_GPL,
    "(c) 2002, MiMaS", 0, WWW_KTT, "ktt@tengwar.art.pl");
  aboutData.addAuthor("MiMaS (Michal Swiatkiewicz)", "", "mimas@tengwar.art.pl");
  aboutData.addCredit("BLiZZARD (Tomas Nemec)", I18N_NOOP("Narmacil transcription engine"));
  
  KCmdLineArgs::init( argc, argv, &aboutData );
  KCmdLineArgs::addCmdLineOptions( options );

  KApplication app;

  if (app.isRestored())
  {
    RESTORE(KTTApp);
  }
  else 
  {
    KCmdLineArgs *args = KCmdLineArgs::parsedArgs();

    Splashscreen* splash = 0L;
    if ( args->isSet("splash") )      // default, unless --nosplash specified
    {
      splash = new Splashscreen();
      splash->show();
    }
    
    KTTApp *ktt = new KTTApp();
    ktt->show();

    if ( splash ) {
      splash->raise();
      QApplication::flushX();
    }    

    QCString fileArg = args->getOption("roman");
    if ( !fileArg.isEmpty() )
    {
      KURL url = fileArg;
      if ( url.protocol().isEmpty() )
        url = QString("file:" + fileArg);
      ktt->openDocumentFile(url, DOCUMENT_TYPE_ROMAN);
    }  

    fileArg = args->getOption("tengwar");
    if ( !fileArg.isEmpty() )
    {
      KURL url = fileArg;
      if ( url.protocol().isEmpty() )
        url = QString("file:" + fileArg);
      ktt->openDocumentFile(url, DOCUMENT_TYPE_TENGWAR);
    }

    fileArg = args->getOption("mode");
    if ( !fileArg.isEmpty() )
    {
      KURL url = fileArg;
      if ( url.protocol().isEmpty() )
        url = QString("file:" + fileArg);
      ktt->loadNarmacilMode(url);
    }
    
    args->clear();

    if ( splash )
      delete splash;
  }

  return app.exec();
}  
